@@ -1,7 +1,7 @@
 %%
 %% %CopyrightBegin%
 %%
-%% Copyright Ericsson AB 1997-2017. All Rights Reserved.
+%% Copyright Ericsson AB 1997-2018. All Rights Reserved.
 %%
 %% Licensed under the Apache License, Version 2.0 (the "License");
 %% you may not use this file except in compliance with the License.
@@ -707,6 +707,7 @@ gen_exports([_|_]=L0, Prefix, Arity) ->
 pgen_dispatcher(Erules, []) ->
     gen_info_functions(Erules);
 pgen_dispatcher(Gen, Types) ->
+    %% MODULE HEAD
     emit(["-export([encode/2,decode/2]).",nl,nl]),
     gen_info_functions(Gen),
 
@@ -714,6 +715,7 @@ pgen_dispatcher(Gen, Types) ->
     NoFinalPadding = lists:member(no_final_padding, Options),
     NoOkWrapper = proplists:get_bool(no_ok_wrapper, Options),
 
+    %% ENCODER
     Call = case Gen of
 	       #gen{erule=per,aligned=true} ->
 		   asn1ct_func:need({per,complete,1}),
@@ -740,6 +742,7 @@ pgen_dispatcher(Gen, Types) ->
     end,
     emit([nl,nl]),
 
+    %% DECODER
     ReturnRest = proplists:get_bool(undec_rest, Gen#gen.options),
     Data = case Gen#gen.erule =:= ber andalso ReturnRest of
 	       true -> "Data0";
@@ -747,6 +750,12 @@ pgen_dispatcher(Gen, Types) ->
 	   end,
 
     emit(["decode(Type, ",Data,") ->",nl]),
+
+    case NoOkWrapper of
+        false -> emit(["try",nl]);
+        true -> ok
+    end,
+
     DecWrap =
 	case {Gen,ReturnRest} of
 	    {#gen{erule=ber},false} ->
@@ -754,32 +763,38 @@ pgen_dispatcher(Gen, Types) ->
 		"element(1, ber_decode_nif(Data))";
 	    {#gen{erule=ber},true} ->
 		asn1ct_func:need({ber,ber_decode_nif,1}),
-		emit(["{Data,Rest} = ber_decode_nif(Data0),",nl]),
+		emit(["   {Data,Rest} = ber_decode_nif(Data0),",nl]),
 		"Data";
 	    {_,_} ->
 		"Data"
 	end,
-    emit([case NoOkWrapper of
-	      false -> "try";
-	      true -> "case"
-	  end, " decode_disp(Type, ",DecWrap,") of",nl]),
-    case Gen of
-	#gen{erule=ber} ->
-	    emit(["  Result ->",nl]);
-	#gen{erule=per} ->
-	    emit(["  {Result,Rest} ->",nl])
-    end,
-    case ReturnRest of
-	false -> result_line(NoOkWrapper, ["Result"]);
-	true ->  result_line(NoOkWrapper, ["Result","Rest"])
+
+    DecodeDisp = ["decode_disp(Type, ",DecWrap,")"],
+    case {Gen,ReturnRest} of
+	{#gen{erule=ber},true} ->
+	    emit(["   Result = ",DecodeDisp,",",nl]),
+            result_line(NoOkWrapper, ["Result","Rest"]);
+	{#gen{erule=ber},false} ->
+	    emit(["   Result = ",DecodeDisp,",",nl]),
+            result_line(NoOkWrapper, ["Result"]);
+
+
+	{#gen{erule=per},true} ->
+	    emit(["   {Result,Rest} = ",DecodeDisp,",",nl]),
+            result_line(NoOkWrapper, ["Result","Rest"]);
+	{#gen{erule=per},false} ->
+	    emit(["   {Result,_Rest} = ",DecodeDisp,",",nl]),
+            result_line(NoOkWrapper, ["Result"])
     end,
+
     case NoOkWrapper of
 	false ->
 	    emit([nl,try_catch(),nl,nl]);
 	true ->
-	    emit([nl,"end.",nl,nl])
+	    emit([".",nl,nl])
     end,
 
+    %% REST of MODULE
     gen_decode_partial_incomplete(Gen),
     gen_partial_inc_dispatcher(Gen),
 
@@ -787,7 +802,7 @@ pgen_dispatcher(Gen, Types) ->
     gen_dispatcher(Types, "decode_disp", "dec_").
 
 result_line(NoOkWrapper, Items) ->
-    S = ["    "|case NoOkWrapper of
+    S = ["   "|case NoOkWrapper of
 		    false -> result_line_1(["ok"|Items]);
 		    true -> result_line_1(Items)
 		end],
@@ -796,12 +811,11 @@ result_line(NoOkWrapper, Items) ->
 result_line_1([SingleItem]) ->
     SingleItem;
 result_line_1(Items) ->
-    ["{",string:join(Items, ","),"}"].
+    ["{",lists:join(",",Items),"}"].
 
 try_catch() ->
     ["  catch",nl,
-     "    Class:Exception when Class =:= error; Class =:= exit ->",nl,
-     "      Stk = erlang:get_stacktrace(),",nl,
+     "    Class:Exception:Stk when Class =:= error; Class =:= exit ->",nl,
      "      case Exception of",nl,
      "        {error,{asn1,Reason}} ->",nl,
      "          {error,{asn1,{Reason,Stk}}};",nl,
@@ -943,7 +957,7 @@ open_hrl(OutFile,Module) ->
 
 hrl_protector(OutFile) ->
     BaseName = filename:basename(OutFile),
-    P = "_" ++ string:to_upper(BaseName) ++ "_HRL_",
+    P = "_" ++ string:uppercase(BaseName) ++ "_HRL_",
     [if
 	 $A =< C, C =< $Z -> C;
 	 $a =< C, C =< $a -> C;
